# TO DO
# - file name and path nrt
# - metadata tags nrt
# - dem path
# - scaling probs
# - centralized inference code

from loguru import logger
import json
import torch
import numpy as np
from rasterio.crs import CRS
from shapely.geometry import Point
from skimage.transform import resize
import geopandas as gpd
from pathlib import Path
import xarray as xr
import enum
import datetime
import os
import requests

# both satio functions are used to load the input data and could be copied
from satio.geoloader import load_reproject4
from veg_workflows.io import save_geotiff

# from evotrain.models.lsc10.v15.labels import (
#     LSC_COVER_LABELS,
#     LSC_OCCLUSION_LABELS,
#     LSC_ECOSYSTEM_LABELS,
# )
from evotrain.models.lsc10.v15.model import Net
from evotrain.models.lsc10.v15.dataset import apply_augmented_scaling
from evotrain.models.lsc10 import day_of_year_cyclic_feats

from climatic_regions import load_meteo_embeddings, lat_lon_to_unit_sphere, load_latlon
import zipfile

S2_RESOLUTION = {
    "B02": "10m",
    "B03": "10m",
    "B04": "10m",
    "B08": "10m",
    "B05": "20m",
    "B06": "20m",
    "B07": "20m",
    "B8A": "20m",
    "B11": "20m",
    "B12": "20m",
    "SCL": "20m",
}
S2_BANDS_10M = ["B02", "B03", "B04", "B08"]
S2_BANDS_20M = ["B05", "B06", "B07", "B8A", "B11", "B12", "SCL"]
S2_BANDS = S2_BANDS_10M + S2_BANDS_20M

LSC_LABELS = [1, 2, 3, 4, 5, 7, 6, 255, 254]
LSC_LEGEND_STR = "0 No data \n1 Tree \n2 Shrubs \n3 Herbaceous vegetation \n4 Not vegetated land \n5 Water \n6 Snow/Ice \n7 Built-up \n254 Shadow \n255 Clouds"

LSC_PROB_BAND_NAMES = [
    "Tree",
    "Shrub",
    "Herbaceous vegetation",
    "Not vegetated land",
    "Water",
    "Built up",
    "Snow/Ice",
    "Clouds",
    "Shadow",
]

LSC_RAW_PROB_BAND_NAMES = [
    "Tree",
    "Shrub",
    "Herbaceous vegetation",
    "Not vegetated land",
    "Water",
    "Snow",
    "Clouds",
    "Shadow",
    "Surface",
    "Cropland",
    "Mangroves",
    "Built",
    "Wetland",
    "Lichens",
    "Other",
]

LSC_BAND_NAMES = {
    "LSC_map": ["MAP"],
    "LSC_probs": LSC_PROB_BAND_NAMES,
    "LSC_raw_probs": LSC_RAW_PROB_BAND_NAMES,
}

DEM_PATH = Path(
    "/vitodata/vegteam/auxdata/dem/COP-DEM_GLO-30-DTED/COP-DEM_GLO-30-DTED.vrt"
)

# ----------------------------------------------------------------------------
# Load HEAD features
# ----------------------------------------------------------------------------


def load_feats_head(bounds, epsg, year, resolution=10):
    meteo_emb = load_meteo_embeddings(
        bounds, epsg, year, bounds_buffer=3000, order=2, resolution=resolution
    )
    latlon = load_latlon(bounds, epsg, resolution=resolution)
    xyz = lat_lon_to_unit_sphere(latlon[0], latlon[1])

    feats_head = np.concatenate((xyz, meteo_emb), axis=0).astype(np.float32)
    return feats_head


# ----------------------------------------------------------------------------
# Load LSF features
# ----------------------------------------------------------------------------


# load the features for a given area
def load_nrt_features(block, bands, resolution):
    prod_path, baseline = (Path(block.path), block.pb_int)
    (bounds, epsg) = (block.bounds, block.epsg)

    list_arr = []
    list_bands = []
    for band in bands:
        if band in S2_BANDS:
            fn = load_file_names(prod_path, band)
            scaling = True
            if band == "SCL":
                scaling = False

            logger.info(f"Loading band: {band} from {fn}")

            b = load_reproject_im(
                fn, bounds, epsg, resolution, baseline, scaling=scaling
            )

            logger.info(f"Loaded band: {band} with shape: {b.shape}")

            list_arr.append(b)
            list_bands.append(band)

        elif band == "cop-DEM-alt":
            # load DEM
            arr_dem = np.squeeze(
                load_reproject4(DEM_PATH, bounds, epsg, resolution=resolution)
            )
            list_arr.append(arr_dem)
            list_bands.append(band)

        else:
            raise ValueError(
                f"Error with band: {band}.\nFailed to extract band: {band}"
            )

    arr = np.stack(list_arr)

    attrs = {"epsg": epsg, "bounds": bounds}

    new_y, new_x = compute_pixel_coordinates(bounds, arr.shape[-2:])

    darr = xr.DataArray(
        arr,
        dims=["band", "y", "x"],
        coords={"band": list_bands, "y": new_y, "x": new_x},
        attrs=attrs,
    )
    return darr.astype(np.float32)


def load_file_names(path, band):
    lsf_res = S2_RESOLUTION[band]
    path = list(path.glob("GRANULE/*/IMG_DATA"))[0]
    fn = path / f"R{lsf_res}"
    fn = list(fn.glob(f"*{band}_{lsf_res}.jp2"))[0]
    return fn


def load_reproject_im(fn, bounds, epsg, resolution, baseline, scaling=True):
    b = np.squeeze(load_reproject4(fn, bounds, epsg, resolution))
    if scaling:
        if baseline > 3:
            b = (b.astype(np.float32) - 1000) / 10000
        else:
            b = (b.astype(np.float32)) / 10000
    return b


def compute_pixel_coordinates(bounds, shape):
    """
    Compute the y and x coordinates for every pixel in an image.

    Args:
    bounds (tuple): A tuple containing (xmin, ymin, xmax, ymax).
    shape (tuple): A tuple containing the image shape (rows, columns).

    Returns:
    tuple: Two arrays containing y and x coordinates for every pixel.
    """
    xmin, ymin, xmax, ymax = bounds
    rows, cols = shape

    x_res = (xmax - xmin) / cols
    y_res = (ymax - ymin) / rows

    if x_res != y_res:
        raise ValueError(
            "Different resolution for y and x axis are not "
            "supported. Bounds and shape are not consistent "
            "with the same resolution on both axis."
        )

    res_half = x_res / 2

    xx = np.linspace(xmin + res_half, xmax - res_half, cols)

    yy = np.linspace(ymax - res_half, ymin + res_half, rows)

    return yy, xx


# ----------------------------------------------------------------------------
# Predict LSC
# ----------------------------------------------------------------------------
def binarize_probs(proba, labels, axis=0):
    """
    Returns classification of probabilities array.
    For each pixel the maximum probability class is selected
    and the appropriate label value from `classes_labels` is
    used.
    """
    # if isinstance(labels[0], str):
    #     labels = get_label_code(labels)
    labels = np.array(labels)
    smooth_labels = np.argmax(proba, axis=axis)

    k = np.arange(labels.size)
    v = labels

    out = np.zeros_like(smooth_labels, dtype=np.uint8)
    for key, val in zip(k, v):
        out[smooth_labels == key] = val

    return out


def load_lsc_net_config(base_models_path, model_version, checkpoint_number):
    model_path = base_models_path / model_version

    config_path = model_path / "config.json"
    with open(config_path, "r") as f:
        config = json.load(f)

    checkpoints = list(model_path.glob("**/*.ckpt"))
    checkpoint_path = [
        c for c in checkpoints if f"epoch={checkpoint_number}" in str(c)
    ][0]

    logger.info(f"Loading model from {checkpoint_path}")

    device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

    net = Net.load_from_checkpoint(checkpoint_path, map_location=device)
    _ = net.eval()
    net = net.to(device)
    return net, config


class LscInferencePipeline:
    def __init__(
        self,
        base_models_path,
        model_version,
        checkpoint_number,
        resolution,
    ):
        self.model_version = model_version
        self.checkpoint_number = checkpoint_number
        self.resolution = resolution
        self.net, self.config = load_lsc_net_config(
            base_models_path=base_models_path,
            model_version=self.model_version,
            checkpoint_number=self.checkpoint_number,
        )

    @property
    def bands(self):
        return self.config["model"]["bands"]

    def load_data(self, block):
        logger.info("loading input data ...")
        year = block.year
        month = block.month
        day = block.day

        # load input features
        feats = load_nrt_features(block, self.bands, resolution=self.resolution)
        feats = feats.sel(band=self.bands)
        feats = apply_augmented_scaling(
            feats,
            k_factor=self.config["dataloader"]["k_factor"],
            k_factor_jitter=0,
        )

        n_bands, n_rows, n_cols = feats.shape

        feats_head = load_feats_head(block.bounds, block.epsg, year)

        sample_id = f"1_{year}{month:02d}{day:02d}"
        doy = day_of_year_cyclic_feats(sample_id, 0, n_rows, n_cols)

        feats_head = np.concatenate((feats_head, doy), axis=0).astype(np.float32)

        scl = load_nrt_features(block, ["SCL"], resolution=self.resolution)

        nodata_mask = scl == 0
        if nodata_mask.any():
            logger.warning("No data in S2 data. Returning NaNs in probs")
            mask = ~np.repeat(nodata_mask, n_bands, axis=0)
            mask = mask.assign_coords(band=feats.band)

            feats = feats.where(mask, 0)

        nodata_mask = nodata_mask.data.squeeze()
        if np.isnan(feats.data).any():
            logger.warning("NaNs found in features. Returning NaNs in probs")
            feats = feats.fillna(0)
            nodata_mask[np.isnan(feats.data).any(axis=0)] = True

        if np.isnan(feats_head).any():
            logger.warning("NaNs found in head features. ")
            feats_head = feats_head.fillna(0)
            # nodata_mask[np.isnan(feats_head.data).any(axis=0)] = True

        logger.info(f"feats: {feats.shape}")
        logger.info(f"feats_head: {feats_head.shape}")
        logger.info(f"nodata_mask: {nodata_mask.shape}")

        return feats, feats_head, nodata_mask

    def predict(self, feats, feats_head):
        logger.info("Running inference... ")

        # check first feature. DEM could be NaN so we don't sum
        nodata_mask = np.isnan(feats.data[0])

        if np.isnan(feats.data).any():
            logger.warning("NaNs found in features. Returning NaNs in probs")
            feats = feats.fillna(0)

        if np.isnan(feats_head).any():
            logger.warning("NaNs found in head features. ")
            feats_head = feats_head.fillna(0)

        with torch.no_grad():
            list_probs_raw = self.net.predict(feats.data, feats_head)

        def mask(probs, nodata_mask):
            probs.squeeze()[:, nodata_mask] = np.nan
            return probs

        return [mask(probs, nodata_mask) for probs in list_probs_raw]

    # def prediction(self, list_probs, nodata_val=0):
    #     list_labels = [
    #         LSC_COVER_LABELS,
    #         LSC_OCCLUSION_LABELS,
    #         LSC_ECOSYSTEM_LABELS,
    #     ]
    #     list_preds = []
    #     for probs, LSC_LABELS in zip(list_probs, list_labels):
    #         nodata_mask = np.isnan(probs).any(axis=0)
    #         pred = binarize_probs(probs, LSC_LABELS)
    #         pred[nodata_mask] = nodata_val
    #         list_preds.append(pred)
    #     return list_preds

    def export_model_to_onnx(
        self,
        dest_folder: Path,
        padding=64,
        opset_version=19,
        upload_to_artifactory=False,
    ):
        """Export the model to ONNX format."""

        padded_dim = 1152
        spectral_dummy_input = torch.randn(
            (1, 11, padded_dim, padded_dim), dtype=self.net.dtype
        )
        feats_head_dummy_input = torch.randn(
            (1, 11, padded_dim, padded_dim), dtype=self.net.dtype
        )
        dummy_input = (spectral_dummy_input, feats_head_dummy_input)

        filename = f"{self.model_version}_onnx_opset_v{opset_version}_static_v2"
        onnx_filename = f"{filename}.onnx"
        onnx_filepath = dest_folder / onnx_filename

        logger.info(f"Exporting model to ONNX format {onnx_filepath}...")
        torch.onnx.export(
            self.net,
            dummy_input,
            f=onnx_filepath,
            input_names=["spectral", "feats_head"],
            output_names=["output"],
            verbose=False,
        )

        zip_filepath = dest_folder / f"{filename}.zip"
        logger.info(f"Zipping ONNX model to {zip_filepath}...")
        with zipfile.ZipFile(zip_filepath, "w") as z:
            # Include onnx file
            z.write(onnx_filepath, arcname=onnx_filename)

            # Include txt file with export settings
            with z.open("export_settings.txt", "w") as f:
                f.write(
                    (
                        f"Model version: {self.model_version}\n"
                        f"Opset version: {opset_version}\n"
                        f"Input names: spectral, feats_head\n"
                        f"Output names: output\n"
                        f"Spectral input shape: {spectral_dummy_input.shape}\n"
                        f"Feats head input shape: {feats_head_dummy_input.shape}\n"
                        f"Padding: {padding}\n"
                        f"Method: static\n"
                    ).encode("UTF-8")
                )

        logger.info("Model exported successfully")

        if upload_to_artifactory:
            upload_zip_to_artifactory(
                zip_filepath,
            )


def upload_zip_to_artifactory(
    zip_filepath: Path, atf_path: str = "auxdata-public/LCFM"
):
    artifactory_username = os.getenv("ARTIFACTORY_USERNAME")
    artifactory_password = os.getenv("ARTIFACTORY_PASSWORD")

    if not artifactory_username or not artifactory_password:
        raise ValueError(
            "Artifactory credentials not found. Please set ARTIFACTORY_USERNAME and ARTIFACTORY_PASSWORD."
        )

    headers = {"Content-Type": "application/octet-stream"}
    upload_url = (
        f"https://artifactory.vgt.vito.be/artifactory/{atf_path}/{zip_filepath.name}"
    )

    logger.info(f"Uploading {zip_filepath} to {upload_url}...")
    with open(zip_filepath, "rb") as f:
        response = requests.put(
            upload_url,
            headers=headers,
            data=f,
            auth=(artifactory_username, artifactory_password),
            timeout=180,
        )

    response.raise_for_status()
    return upload_url


# postprocess the probabilities into:
# merged_probs: the merged probabilities of the components
# pred_lsc: the classification map
# rgb_probs: an RGB with the merged class probabilities
# rgb_ecosystem: an RGB of the ecosystem component probabilities
def post_process(list_probs, nodata_mask):
    logger.info("postprocessing ...")

    # stack the probabilities
    raw_probs = np.hstack(list_probs).squeeze()

    # merge the probabilities of the components
    merged_probs = merge_probs(list_probs)

    # get the classification map
    pred_lsc = binarize_probs(merged_probs, LSC_LABELS)

    merged_probs[:, nodata_mask] = np.nan
    raw_probs[:, nodata_mask] = np.nan
    pred_lsc[nodata_mask] = 0

    return merged_probs, pred_lsc, raw_probs

def merge_probs(list_probs):
    prob_surface = list_probs[1].squeeze()[-1, ...]
    probs_cover = list_probs[0].squeeze().copy()
    prob_built = list_probs[2].squeeze()[2, ...].copy()
    prob_notveg_raw = list_probs[0].squeeze()[3, ...].copy()

    # split the not-vegetated class into not-vegetated and built-up
    probs_cover[3, ...] = prob_notveg_raw * (np.ones(prob_built.shape) - prob_built)
    prob_built = prob_notveg_raw * prob_built
    probs_cover = np.vstack([probs_cover, prob_built[np.newaxis, ...]])

    # add occlusion classes
    probs_normalized = probs_cover / probs_cover.sum(axis=0) * prob_surface
    probs_normalized = np.vstack(
        [
            probs_normalized,
            list_probs[1].squeeze()[:-1, ...],  # prob_occlusion[np.newaxis, ...]
        ]
    )

    # set probability of nodata to 0
    nodata_mask = np.isnan(probs_normalized).any(axis=0)
    probs_normalized[:, nodata_mask] = 0

    return probs_normalized


# ----------------------------------------------------------------------------
# Colormap and tags
# ----------------------------------------------------------------------------


class LSCContinuousLabelsColors(enum.Enum):
    TREE = (
        1,
        "tree",
        "tree",
        "Trees covered area",
        np.array([15, 100, 15]) / 255,  # np.array([0, 100, 0]) / 255,
    )
    SHRUB = (
        2,
        "shrub",
        "shrub",
        "Shrub cover area",
        np.array([255, 197, 67]) / 255,  # np.array([255, 187, 34]) / 255,
    )
    HERBACEOUS = (
        3,
        "herbaceous",
        "herbaceous",
        "Herbaceous",
        np.array([255, 255, 103]) / 255,  # np.array([255, 255, 76]) / 255,
    )
    NOT_VEGETATED = (
        4,
        "not vegetated",
        "not vegetated",
        "Not vegetated",
        np.array([180, 180, 180]) / 255,  # np.array([180, 180, 180]) / 255)
    )
    WATER = (
        5,
        "water",
        "water",
        "Water",
        np.array([30, 115, 200]) / 255,  # np.array([0, 100, 200]) / 255,
    )
    SNOW = (
        6,
        "snow and ice",
        "snow and ice",
        "snow and ice",
        np.array([240, 240, 240]) / 255,  # np.array([240, 240, 240]) / 255,
    )
    BUILT = (
        7,
        "built",
        "urban/built-up",
        "Built-up",
        np.array([250, 38, 38]) / 255,  # np.array([250, 0, 0]) / 255,
    )
    CLOUD = (
        255,
        "cloud",
        "cloud",
        "cloud",
        np.array([194, 88, 232]) / 255,  # np.array([184, 2, 129]) / 255,
    )
    SHADOW = (
        254,
        "shadow",
        "shadow",
        "shadow",
        np.array([124, 56, 148]) / 255,
    )
    NO_DATA = (0, "nodata", "Not sure", "No Data", np.array([0, 0, 0]))

    def __init__(self, val1, val2, val3, val4, val5):
        self.id = val1
        self.class_name = val2
        self.iiasa_name = val3
        self.esa_class_name = val4
        self.color = val5


def get_labels_encodings(col_enum):
    return [
        {
            "id": lc.id,
            "color": lc.color,
            "iiasa_name": lc.iiasa_name,
            "name": lc.class_name,
            "esa_name": lc.esa_class_name,
        }
        for lc in col_enum
    ]


class LSCEcosystemLabelsColors(enum.Enum):
    CROP = (1, "crop", "crops", "Cropland", np.array([240, 150, 255]) / 255)
    MANGROVES = (
        2,
        "mangroves",
        None,
        "Mangroves",
        np.array([0, 207, 117]) / 255,
    )
    BUILT = (
        3,
        "built",
        "urban/built-up",
        "Built-up",
        np.array([250, 0, 0]) / 255,
    )

    WETLAND = (
        4,
        "wetland",
        "wetland (herbaceous)",
        "Herbaceous wetland",
        np.array([0, 150, 160]) / 255,
    )
    LICHENS = (
        5,
        "lichens_mosses",
        "Lichen and moss",
        "Lichen and moss",
        np.array([250, 230, 160]) / 255,
    )
    OTHER = (
        6,
        "other",
        "other",
        "Other",
        np.array([255, 255, 255]) / 255,  # 0, 150, 0
    )

    NO_DATA = (0, "nodata", "Not sure", "No Data", np.array([0, 0, 0]))

    def __init__(self, val1, val2, val3, val4, val5):
        self.id = val1
        self.class_name = val2
        self.iiasa_name = val3
        self.esa_class_name = val4
        self.color = val5


LSC_ENCODINGS = get_labels_encodings(LSCContinuousLabelsColors)
LSC_ENCODINGS_DICT = {e["id"]: e for e in LSC_ENCODINGS}
LSC_COLORMAP = {
    e["id"]: (e["color"] * 255).astype(int).tolist() + [255] for e in LSC_ENCODINGS
}

LSC_ECO_ENCODINGS = get_labels_encodings(LSCEcosystemLabelsColors)
LSC_ECO_ENCODINGS_DICT = {e["id"]: e for e in LSC_ECO_ENCODINGS}
LSC_ECO_COLORMAP = {
    e["id"]: (e["color"] * 255).astype(int).tolist() + [255] for e in LSC_ECO_ENCODINGS
}


def last_day_of_month(year, month):
    # The day 28 exists in every month. 4 days later, it's always next month
    next_month = datetime.date(year, month, 28) + datetime.timedelta(days=4)
    # subtracting the number of the current day brings us back one month
    return next_month - datetime.timedelta(days=next_month.day)


def get_band_tags(product_type):
    return [{f"B{ind}": name} for ind, name in enumerate(LSC_BAND_NAMES[product_type])]


def get_tags(block, product_version, product_type, resolution, frequency="monthly"):
    tags = {
        "copyright": f"Contains modified Copernicus Sentinel data ({block.year})",
        "creation_time": str(datetime.datetime.now()),
        "license": "CC-BY 4.0 - https://creativecommons.org/licenses/by/4.0/",
        "product_crs": f"EPSG:{block.epsg}",
        "product_grid": "Sentinel-2 tiling grid",
        "product_tile": f"{block.tile}",
        "product_type": product_type,
        "product_version": product_version,
        "reference": "this_should_be_an_url",
        "band_names": LSC_BAND_NAMES[product_type],
    }

    if frequency == "monthly":
        last_day = last_day_of_month(block.year, block.month).day
        tags["time_end"] = (f"{block.year}-{block.month:02d}-01T23:59:59Z",)
        tags["time_start"] = (
            f"{block.year}-{block.month:02d}-{last_day:02d}T00:00:00Z",
        )
        tags["title"] = (
            f"LSC product at {resolution}m resolution for {block.year}-{block.month:02d}-01 - {block.year}-{block.month:02d}-{last_day:02d}",
        )
    elif frequency == "nrt":
        tags["time_end"] = (
            f"{block.year}-{block.month:02d}-{block.day:02d}T23:59:59Z",
        )
        tags["time_start"] = (
            f"{block.year}-{block.month:02d}-{block.day:02d}T00:00:00Z",
        )
        tags["title"] = (
            f"LSC product at {resolution}m resolution for {block.year}-{block.month:02d}-{block.day:02d}",
        )

    if product_type == "LSC_map":
        tags["legend"] = (LSC_LEGEND_STR,)
    return tags


# ----------------------------------------------------------------------------
# Main process functions
# ----------------------------------------------------------------------------


def write_output(
    lsc_coll,
    block,
    merged_probs,
    pred_lsc,
    raw_probs,
    nodata_mask,
    resolution,
    frequency,
):
    logger.info("writing output ...")

    lsc_block_product = lsc_coll.block(
        block.tile, block.block_id, year=block.year, month=block.month
    )

    if merged_probs is not None:
        merged_probs = merged_probs.squeeze() * 100
        merged_probs[:, nodata_mask] = 255

        fn = lsc_block_product.path("PROB")
        fn.parent.mkdir(parents=True, exist_ok=True)
        save_geotiff(
            (merged_probs).astype(np.uint8),
            filename=fn,
            bounds=block.bounds,
            epsg=block.epsg,
            tags=get_tags(
                block,
                product_version=lsc_coll.version,
                product_type="LSC_probs",
                resolution=resolution,
                frequency=frequency,
            ),
            bands_names=LSC_BAND_NAMES["LSC_probs"],
            band_tags=get_band_tags("LSC_probs"),
            colormap=None,
            nodata_value=255,
        )

    if raw_probs is not None:
        raw_probs = raw_probs.squeeze() * 100
        raw_probs[:, nodata_mask] = 255

        fn = lsc_block_product.path("PROB-RAW")
        fn.parent.mkdir(parents=True, exist_ok=True)
        save_geotiff(
            (raw_probs).astype(np.uint8),
            filename=fn,
            bounds=block.bounds,
            epsg=block.epsg,
            tags=get_tags(
                block,
                product_version=lsc_coll.version,
                product_type="LSC_raw_probs",
                resolution=resolution,
                frequency=frequency,
            ),
            bands_names=LSC_BAND_NAMES["LSC_raw_probs"],
            band_tags=get_band_tags("LSC_raw_probs"),
            colormap=None,
            nodata_value=255,
        )

    if pred_lsc is not None:
        pred_lsc[nodata_mask] = 0

        fn = lsc_block_product.path("MAP")
        fn.parent.mkdir(parents=True, exist_ok=True)
        save_geotiff(
            (pred_lsc.squeeze()).astype(np.uint8),
            filename=fn,
            bounds=block.bounds,
            epsg=block.epsg,
            tags=get_tags(
                block,
                product_version=lsc_coll.version,
                product_type="LSC_map",
                resolution=resolution,
                frequency=frequency,
            ),
            bands_names=LSC_BAND_NAMES["LSC_map"],
            band_tags=get_band_tags("LSC_map"),
            colormap=LSC_COLORMAP,
            nodata_value=0,
        )
